#
# SSM_Tools::Ini -- SSM_Tools Ini File parsing
#
# Copyright (C) 2017, 2018 Francesco P. Lovergine <francesco.lovergine@cnr.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2, 
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

#
# See POD at the __END__ for documentation
#
package SSM_Tools::Ini;

use v5.18;

use strict;

use warnings;
no warnings 'experimental::smartmatch';

use Carp;
use Config::IniFiles;

require Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

# set the version for version checking
$VERSION     = '0.02';

@ISA         = qw(Exporter);
@EXPORT      = qw();

%EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],

# your exported package globals go here,
# as well as any optionally exported functions
@EXPORT_OK   = qw();

# non-exported package globals go here
use vars qw();

# all file-scoped lexicals must be created before
# the functions below that use them.

# file-private lexicals go here

sub new($$) {
	my ($class, $inifile) = @_;
	$inifile //= "/usr/local/sm-tools/etc/sm.ini";
	
	my $self = {};
	
	my ($cfg, $root, $database, $repoid, $repopath) = (undef, undef, undef, undef, undef);
	eval {
		$cfg = Config::IniFiles->new(-file => $inifile);
		$root = $cfg->val('General','Root');
		$database = $cfg->val('General','Database');
		$repoid = $cfg->val('General','RepositoryId');
		$repopath = $cfg->val('General','RepositoryPath');
	};
	$self->{ini} = $cfg;
	$self->{root} //= $root;
	$database //= "/usr/local/sm-tools/data/es1.splite";
	$repopath //= "/usr/local/sm-tools/data/repository/0";
	$self->{database} = $database;
	$self->{repoid} = $repoid;
	$self->{repopath} = $repopath;
	bless $self, $class;
	return $self;
}

sub get_root($) {
	my $self = shift;
	$self->{root};
}

sub get_database($) {
	my $self = shift;
	$self->{database};
}

sub get_repo_id($) {
	my $self = shift;
	$self->{repoid};
}

sub get_repo_path($) {
	my $self = shift;
	$self->{repopath};
}

sub get($$$) {
	my ($self, $section, $key) = @_;
	return $self->{ini}->val($section, $key) if defined($self->{ini});
	undef;
}

END { }       # module clean-up code here (global destructor)

1;

__END__

=head1 NAME

SSM_Tools::Ini -- a module to parse the general INI file for SM-Tools programs.

=head1 SYNOPSIS

  use SSM_Tools::Ini;

  $ini = SSM_Tools;:Ini->new();
  $ini = SSM_Tools;:Ini->new($ini_path);

  $ini->get_root;
  $ini->get_database;
  $ini->get_repo_id;
  $ini->get_repo_path;
  $ini->get($section, $keyword);

=head1 DESCRIPTION

This simple module is used to open and read the INI file used by most
of the SM-Tools utilities. 

=head2 Class Variables and Methods

A list with descriptions of the module public methods follows. Note
that the package provides only a full OO interface, no exported subs.

=head3 SSM_Tools;:Ini->new($filename)

Returns an object that can be used to get (Section,Keyword) attributes
present in the inifile. The $filename parameter is optional: if missing
the module will use the default file name. If an ini file cannot
be accessed or correctly read, the module will return a diagnostic
message and continues. Managing errors is a task of the caller.

=head3 $ini->get_root

Returns the B<Root> value as eventually set in the B<General> section. The Root
is a path prefixed to all external rasters referenced in the database. 
It is optional (if not specified paths will be considered as they are) 
but each tool can override that value if required. It is not a good
idea anyway, because using a single tree for all raster files is the
better approach, but that approach can be exploited when the same
tree is used on multiple machines with different mount points.

=head3 $ini->get_database

Returns the B<Database> value as eventually set in the B<General> section.
The Database is the full path name of the Spatialite database used by
all tools. The default path is /usr/local/sm-tools/data/es1.splite for
historical reasons.

=head3 $ini->get_repo_id

Returns the B<RepositoryId> value as eventually set in the B<General> section.
The Repository ID is the index in the Path table the addresses the full 
path of the default repository. That directory should already exist
or it will be created at its first use. The default is undefined and
will trigger the creation of a new folder as specified in RepositoryPath.

=head3 $ini->get_repo_path

Returns the B<RepositoryPath> value as eventually set in the B<General> section.
The Repository Path is the full path name of the default repoository. It should
already exist or it will be created at its first use (along with the corresponding ID). 
The default is /usr/local/sm-tools/data/repository/0 for historical reasons. That could
be the wrong choice when detaching in link mode your raster files.

=head3 $db->get($section, $keyword)

Returns the value assigned to a certain Keyword in a specific Section, 
both passed as arguments for the method.


=head1 SEE ALSO

See the Config::IniFiles module for complete information about the INI file parsing.

=head1 AUTHOR

Francesco P. Lovergine

=head1 COPYRIGHT

Copyright (C) 2017-2021 Francesco P. Lovergine <francesco.lovergine@cnr.it>
 
=cut
