#!/bin/bash
#
# What follows is a specific selection of ~520m grid in the area of
# Segezia used to group together sensors and get statistics. This script
# can be sourced to define dynamically grids (G[1..7]) in WKT form and
# sensors names (P[1..7]) and IDs (S[1..7]) logically grouped together.
#
# This file is based on a grid created by projecting in WGS84 the
# original UTM33 Segezia grid. The original grid is the result of the
# following operations:
#
# g.region rast=segezia_map_520m
# g.region vect=segezia_network -a
# g.region n=n+520 e=e+520 res=520 -p
# v.mkgrid pos=region map=segezia_network_smosar_grid_520m grid=6,6 --o
# 
# where segezia_network is the vector of all sensors locations and
# segezia_map_520m is the output of a SMOSAR map. The resulting grid has
# been enlarged in north/east directions to add one more row and col to
# the minimal grid that includes all sensors.
#
# Note that the choice of cells G1..G7 has been done by hand to
# include 1-4 sensors distributed per cell.
#

pushd $SM_TOOLS_OUTPUTDIR >/dev/null

declare -a CAT POLY G P S 

case "$1" in
	asc)
		NCATS=5
		dir=$1
		CAT[1]="26 27 33 34"
		CAT[2]="17 18 24 25"
		CAT[3]="15 16 22 23"
		CAT[4]="3 4 10 11"
		CAT[5]="1 2 8 9"
		OK=yes
		;;
	desc)
		NCATS=4
		dir=$1
		CAT[1]="19 20 26 27"
		CAT[2]="16 17 23 24"
		CAT[3]="3 4 10 11"
		CAT[4]="1 2 8 9"
		OK=yes
		;;
	ascdesc)
		NCATS=4
		dir=$1
		CAT[1]="19 20 26 27"
		CAT[2]="16 17 23 24"
		CAT[3]="3 4 10 11"
		CAT[4]="1 2 8 9"
		OK=yes
		;;
	*)
		echo "usage: $0 asc|desc|ascdesc"
		;;
esac

GRID=segezia_network_smosar_grid_520m_${dir}.shp

if [ ! -f $GRID ]
then
	echo "Missing grid file $GRID"
	OK=""
fi

if [ ! -z $OK ]
then
	# Run in standard WGS84 CRS to get related sensors
	for i in $(seq 1 $NCATS)
	do
		cats=$(echo ${CAT[i]}|sed -e 's/ /,/g') 
		POLY[$i]=$(ogrinfo -al $GRID -where "cat in ($cats)"|grep POLYGON|sed -e 's/$/,/' |perl -00pe 's/,(?!.*,)//s')
		G[i]=$(spatialite -batch -noheader $SM_TOOLS_DB "select ST_AsText(ST_Envelope(ST_GeomFromText('GEOMETRYCOLLECTION(${POLY[i]})')));")
		P[i]=$(spatialite -batch -noheader $SM_TOOLS_DB "select name from Sensor where ST_Intersects(location,ST_GeomFromText('${G[i]}'));")
		S[i]=$(spatialite -batch -noheader $SM_TOOLS_DB "select id from Sensor where ST_Intersects(location,ST_GeomFromText('${G[i]}'));")
	done
	case $dir in
		asc)
			# Relocate one station missing
			P[1]="${P[1]} em27171"
			S[1]="${S[1]} 10"
			;;
		desc)
			# Relocate one station missing
			P[1]="${P[1]} em27173"
			S[1]="${S[1]} 6"
			;;
		ascdesc)
			# Reloacatone station missing
			P[1]="${P[1]} em27173 em27171"
			S[1]="${S[1]} 6 10"
			;;
	esac
fi

popd >/dev/null
