#!/bin/sh
#
# This simple script computes raster stats against a field map and
# outputs them as different rasters with the same geometry. It combines output
# in a hybrid resolution, starting from a low-res image and an high-res one.
# 

if [ $# -ne 4 ]
then
	echo "usage: $0 <rasterized fields map> <raster list> <output hyb dir> <version>"
	exit 1
fi

$SM_TOOLS/etc/grass_check.sh || exit 1

set -e

FIELD=$1
F=$(basename $FIELD)
LIST=$2
OUTDIR_HYB=$3
VER=$4
REPORT=$(mktemp -q)
RECLASS=$(mktemp -q)
RES=0.0005

mkdir -p $OUTDIR_HYB || true

$G g.mapsets operation=add mapset=SM_TOOLS
$G r.mask -r || true

$G r.in.gdal in=$FIELD out=$F --o
$G g.region -a rast=$F
$G i.segment.stats -c map=$F rasters=$F raster_statistics= area_measures=area csvfile=$REPORT --o
tail -n +2 $REPORT|awk -F\| '{print $1,"=",$2}' >$RECLASS
$G r.reclass in=$F rules=$RECLASS out=zs_tot --o 
$G r.mapcalc expression="polygons=if($F,0,null())" --o

for line in $(cat $LIST|egrep -v '^#')
do
	# line is: lowres_map_prefix,highres_map,output_prefix,reference_date
	target1=$(echo $line|cut -d, -f2)
	target2=$(echo $line|cut -d, -f1)
	output=$(echo $line|cut -d, -f3)
	T1=$(basename $target1)
	T2=$(basename $target2)
	O=$(basename $output)
	DATE=$(echo $line|cut -d, -f4|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')

	$G r.external in=$target1 out=$T1 --o
	$G r.external in=${target2}_mean out=${T2}_mean --o
	$G r.external in=${target2}_stddev out=${T2}_stddev --o
	$G r.external in=${target2}_r out=${T2}_perc --o

	$G g.region -a rast=$F
	$G r.mask raster=$T1 --o
	$G i.segment.stats -c map=$F rasters=$T1 raster_statistics=mean,stddev area_measures=area csvfile=$REPORT --o
	if [ $(cat $REPORT|wc -l) -le 1 ]
	then 
		echo "Missing stats for provided segments. Skipped hi-res."
		HIRES=0
	else
		HIRES=1
	fi

	if [ $HIRES -eq 1 ] 
	then
		$G g.region -a rast=$T1 res=$RES
		tail -n +2 $REPORT|awk -F\| '{print $1,"=",$2}' >$RECLASS
		$G r.reclass in=$F rules=$RECLASS out=zs_area --o 

		tail -n +2 $REPORT|awk -F\| '{print $1,"=",$3}' >$RECLASS
		$G r.reclass in=$F rules=$RECLASS out=zs_mean --o 
		
		tail -n +2 $REPORT|awk -F\| '{print $1,"=",$4}' >$RECLASS
		$G r.reclass in=$F rules=$RECLASS out=zs_stddev --o 

		$G r.mask -r
		$G r.mapcalc expression="zs_perc=round(100.0*(float(zs_area)/float(zs_tot)))" --o
		$G r.mapcalc expression="zs_mean_33=if(zs_perc>=33,zs_mean,0)" --o
		$G r.mapcalc expression="zs_stddev_33=if(zs_perc>=33,zs_stddev,0)" --o

		$G r.patch in=zs_mean_33,polygons,${T2}_mean out=hyb_mean --o
		$G r.patch in=zs_stddev_33,polygons,${T2}_stddev out=hyb_stddev --o
		$G r.patch in=zs_perc,polygons,${T2}_perc out=hyb_perc --o
	else
		$G r.mask -r
		$G g.region -a rast=$T1 res=$RES
		$G r.resample in=${T2}_mean out=hyb_mean --o
		$G r.resample in=${T2}_stddev out=hyb_stddev --o
		$G r.resample in=${T2}_perc out=hyb_perc --o
		$G r.resample in=${T2}_mean out=zs_mean_33 --o
		$G r.resample in=${T2}_stddev out=zs_stddev_33 --o
		$G r.resample in=${T2}_perc out=zs_perc --o
	fi

	$G r.mapcalc expression="hyb_cofvar=10000.0*hyb_stddev/abs(hyb_mean)" --o
	for r in hyb_perc hyb_mean hyb_stddev hyb_cofvar
	do
		$G r.null null=0 map=$r 
	done

    $G r.colors map=hyb_mean rules=$SM_TOOLS/etc/colors.txt
    $G g.region rast=hyb_mean
	$G r.out.png -w -t in=hyb_mean out=${OUTDIR_HYB}/${O}_zs_mean.png compress=9 --o
    $G r.colors map=hyb_stddev rules=$SM_TOOLS/etc/colors_stddev.txt
    $G g.region rast=hyb_stddev
	$G r.out.png -w -t in=hyb_stddev out=${OUTDIR_HYB}/${O}_zs_stddev.png compress=9 --o

	$G i.group in=hyb_mean,hyb_stddev,hyb_cofvar group=hyb --o
	#$G r.out.gdal format=ENVI in=hyb out=${OUTDIR_HYB}/${O} nodata=0 createopt="SUFFIX=ADD" --o 
	$G r.out.gdal -c in=hyb out=${OUTDIR_HYB}/${O}_${VER}.tif nodata=0 --o 
	mv ${OUTDIR_HYB}/${O}_zs_mean.png ${OUTDIR_HYB}/${O}_mean.png
	mv ${OUTDIR_HYB}/${O}_zs_mean.wld ${OUTDIR_HYB}/${O}_mean.wld
	mv ${OUTDIR_HYB}/${O}_zs_stddev.png ${OUTDIR_HYB}/${O}_stddev.png
	mv ${OUTDIR_HYB}/${O}_zs_stddev.wld ${OUTDIR_HYB}/${O}_stddev.wld

	cp 	$SM_TOOLS/etc/ssm-style.css $SM_TOOLS/etc/logo-cnr.png $SM_TOOLS/etc/logo-sensagri.png \
		$SM_TOOLS/etc/colorbar.png $SM_TOOLS/etc/colorbar_stddev.png \
		$SM_TOOLS/etc/*.sld $SM_TOOLS/etc/*.qml \
		$OUTDIR_HYB/$D/.
	sed -e "s/@MEAN@/${O}_mean.png/" \
		-e "s/@STDDEV@/${O}_stddev.png/" \
		-e "s/@TITLE@/H2020 project No. 730074/" \
		-e "s/@PROJ@/logo-sensagri.png/" \
		-e "s/@CNR@/logo-cnr.png/" \
		-e "s/@DATE@/$DATE/" \
		$SM_TOOLS/etc/ssm-template.html \
		>$OUTDIR_HYB/$D/${O}.html

	$G g.remove t=group name=hyb -f

done

rm -f $REPORT $RECLASS

$G_CLEAN

exit 0
