#!/bin/sh
#
# This scripts creates a native resolution vector and raster starting from
# stats of SMOSAR over SMAP standard grids. All products use the native EASE2
# SRS and resolutions.
#

set -e

if [ $# -lt 4 ]
then
	echo "usage: $0 <report> <resolution> <track> <output_dir>"
	exit 1
fi

$SM_TOOLS/etc/grass_check.sh || exit 2

REPORT=$1
RES=$2
TRACK=Medi_$3
PREFIX=$4

if [ ! -d "$PREFIX" ]
then
	mkdir -p "$PREFIX"
fi

case $RES in
	36)
		res=36000
		;;
	25)
		res=25000
		;;
	12d5)
		res=12500
		;;
	9)
		res=9000
		;;
	3)
		res=3000
		;;
	1)
		res=1000
		;;
	*)
		echo "Resolution not valid: $RES"
		exit 3
		;;
esac

grid=smap_grid_${RES}km_${TRACK}
if [ $($G_EASE2 g.list t=vect pat="$grid"|wc -l) -eq 0 ]
then
	echo "Missing working SMAP grid $grid"
	exit 4
fi

SQLFILE=$(mktemp -q)

# parse report line by line and fill a copy of SMAP grid vector
for line in $(tail -n +1 $REPORT)
do
	cat=$(echo $line|cut -d\| -f2)
	corr=$(echo $line|cut -d\| -f4)
	unbrmse=$(echo $line|cut -d\| -f5)
	grid_tmp=smosar_corr_${grid}
	if [ $($G_EASE2 g.list t=vect pat="$grid_tmp"|wc -l) -eq 0 ] 
	then
		$G_EASE2 g.copy vect=$grid,$grid_tmp --o
		$G_EASE2 v.db.addcolumn map=$grid_tmp col="corr double precision,unbrmse double precision" --o
	else
		if [ $($G_EASE2 db.columns table=$grid_tmp|grep corr|wc -l) -eq 0 ]
		then
			$G_EASE2 v.db.addcolumn map=$grid_tmp col="corr double precision,unbrmse double precision" --o
		fi
	fi
	echo "UPDATE $grid_tmp SET corr=$corr,unbrmse=$unbrmse WHERE cat=$cat;" >>$SQLFILE
done

# execute a batch update for all vectors
$G_EASE2 db.execute input=$SQLFILE

# output of the resulting vectors and their rasterized formats (only # mean/10000)
for v in $($G_EASE2 g.list t=vect pat="smosar_corr_${grid}")
do
	$G_EASE2 v.out.ogr in=$v type=area out=$PREFIX/${v}.shp --o
	$G_EASE2 g.region vect=$v res=$res
	$G_EASE2 v.to.rast in=$v out=${v}_corr type=area --o use=attr attr='corr' mem=10000 --o
	$G_EASE2 v.to.rast in=$v out=${v}_unbrmse type=area --o use=attr attr='unbrmse' mem=10000 --o
	$G_EASE2 r.null map=${v}_corr setnull=0
	$G_EASE2 r.null map=${v}_unbrmse setnull=0
	$G_EASE2 r.out.gdal -c -f in=${v}_corr out=$PREFIX/${v}_corr.tif type=Float32 --o 
	$G_EASE2 r.out.gdal -c -f in=${v}_unbrmse out=$PREFIX/${v}_unbrmse.tif type=Float32 --o 
done

rm -f $SQLFILE

$G_CLEAN



