#!/bin/sh
#
# This scripts creates a native resolution vector and raster starting from
# stats of SMOSAR over SMAP standard grids. All products use the native EASE2
# SRS and resolutions.
#

set -e

if [ $# -lt 5 ]
then
	echo "usage: $0 <report> <resolution> <track> <percentage> <output_dir>"
	exit 1
fi

$SM_TOOLS/etc/grass_check.sh || exit 2

REPORT=$1
RES=$2
TRACK=Medi_$3
PERC=$4

PREFIX=$5
if [ ! -d "$PREFIX" ]
then
	mkdir -p "$PREFIX"
fi

case $RES in
	36)
		res=36000
		;;
	25)
		res=25000
		;;
	12d5)
		res=12500
		;;
	9)
		res=9000
		;;
	3)
		res=3000
		;;
	1)
		res=1000
		;;
	*)
		echo "Resolution not valid: $RES"
		exit 3
		;;
esac

grid=smap_grid_${RES}km_${TRACK}
if [ $($G_EASE2 g.list t=vect pat="$grid"|wc -l) -eq 0 ]
then
	echo "Missing working SMAP grid $grid"
	exit 4
fi

SQLFILE=$(mktemp -q)

# parse report line by line and fill a copy of SMAP grid vector
for line in $(tail -n +2 $REPORT|awk -F\| "{if (\$5 >= $PERC) print}")
do
	date=$(echo $line|cut -d\| -f1|sed -e 's/-//g')
	avg=$(echo $line|cut -d\| -f6|awk '{printf "%.6f", $1/10000}')
	cat=$(echo $line|cut -d\| -f2)
	grid_tmp=smosar_${grid}_${date}
	if [ $($G_EASE2 g.list t=vect pat="$grid_tmp"|wc -l) -eq 0 ] 
	then
		$G_EASE2 g.copy vect=$grid,$grid_tmp --o
		$G_EASE2 v.db.addcolumn map=$grid_tmp col="smosar double precision" --o
	else
		if [ $($G_EASE2 db.columns table=$grid_tmp|grep smosar|wc -l) -eq 0 ]
		then
			$G_EASE2 v.db.addcolumn map=$grid_tmp col="smosar double precision" --o
		fi
	fi
	echo "UPDATE $grid_tmp SET smosar=$avg WHERE cat=$cat;" >>$SQLFILE
done

# execute a batch update for all vectors
$G_EASE2 db.execute input=$SQLFILE

# output of the resulting vectors and their rasterized formats (only # mean/10000)
for v in $($G_EASE2 g.list t=vect pat="smosar_${grid}_*")
do
	$G_EASE2 v.out.ogr in=$v type=area out=$PREFIX/${v}.shp --o
	$G_EASE2 g.region vect=$v res=$res
	$G_EASE2 v.to.rast in=$v out=$v type=area --o use=attr attr='smosar' mem=10000 --o
	$G_EASE2 r.null map=$v setnull=0
	$G_EASE2 r.out.gdal -c -f in=$v out=$PREFIX/$v.tif type=Float32 --o 
done

rm -f $SQLFILE

$G_CLEAN



