#!/bin/sh

#
# This scripts computes S1 sigma0 temporal mean values masked by thresholded SMAP value (< min and > max) 
#

set -e

# The following values could be changed for matching specific TAGS and thresholds (site specific)
S1_TAG="sigma0_vv_gtc_tfilt_masked_asc"
THMIN=0.08
THMAX=0.36
REFERENCE="sameday"
#REFERENCE="dayprior"

if [ $# -ge 1 -a $# -lt 4 ]
then
	echo "usage $0: <s1_tag> <thmin> <thmax> sameday|priorday"
	exit 1
else
	if [ $# -gt 1 ]
	then
		S1_TAG="$1"
		THMIN="$2"
		THMAX="$3"
		REFERENCE="$4"
	fi
fi

OUTFILE=$(mktemp -q)
MAPCALC=$(mktemp -q)

SMAP_TAG="smap36"

if [ "$REFERENCE" = "sameday" ]
then
# Sentinel-1 series versus SMAP 36km, same day
sql="select p.filename, p.reference_datetime, m.filename, m.reference_datetime
from ProcessedImage as p, Product as m where
p.tag='$S1_TAG' and
(date(p.reference_datetime)=date(m.reference_datetime) or
date(p.reference_datetime)=date(m.reference_datetime,'+1 day'))and
cast((strftime('%s',p.reference_datetime)-strftime('%s',m.reference_datetime))
as real)/60/60 <= 12 and
cast((strftime('%s',p.reference_datetime)-strftime('%s',m.reference_datetime))
as real)/60/60 > 0 and m.tag='$SMAP_TAG';"
else
REFERENCE="priorday"
# Sentinel-1 series versus SMAP 36km, 1 day before
sql="select p.filename, p.reference_datetime, m.filename, m.reference_datetime
from ProcessedImage as p, Product as m where
p.tag='$S1_TAG' and
(date(p.reference_datetime)=date(m.reference_datetime,'+1 day'))and
cast((strftime('%s',p.reference_datetime)-strftime('%s',m.reference_datetime))
as real)/60/60 <= 36 and m.tag='$SMAP_TAG';"
fi

spatialite -noheader $SM_TOOLS_DB "$sql" | sed -e 's/ /_/g' >$OUTFILE
for line in $(cat $OUTFILE)
do
	s1file=$(echo $line|cut -d\| -f1)
	1datetime=$(echo $line|cut -d\| -f2)
	s1date=$(echo $s1datetime|cut -d_ -f1)
	s1time=$(echo $s1datetime|cut -d_ -f2)
	smapfile=$(echo $line|cut -d\| -f3)
	smap=$(echo $smapfile|sed -e 's/-/_/g')
	smapdatetime=$(echo $line|cut -d\| -f4)
	smapdate=$(echo $smapdatetime|cut -d_ -f1)
	smaptime=$(echo $smapdatetime|cut -d_ -f2)
	$G r.external in=$s1file out=$s1 --o
	$G r.null setnull=0 map=$s1 || true
	$G g.region rast=$s1 
	$G v.mkgrid grid=1,1 pos=region map=reg_$s1 --o
	$G_EASE2 r.external in=$SM_TOOLS_REPO/$smapfile out=$smap --o
	$G_EASE2 r.null setnull=-32768,-32767 map=$smap --o || true
	$G_EASE2 v.proj location=WGS84 input=reg_$s1 output=reg_$s1 --o
	$G_EASE2 g.region rast=$smap
	$G_EASE2 g.region vect=reg_$s1
	cat >$MAPCALC <<EOF
${smap}_thmin=if($smap<=$THMIN,$smap,0)
${smap}_thmax=if($smap>=$THMAX,$smap,0)
${smap}_mask_thmin=if($smap<=$THMIN,1,0)
${smap}_mask_thmax=if($smap>=$THMAX,1,0)
EOF
	$G_EASE2 r.mapcalc file=$MAPCALC --o
	$G_EASE2 r.null null=0 map=${smap}_thmin
	$G_EASE2 r.null null=0 map=${smap}_thmax
	$G_EASE2 r.null null=0 map=${smap}_mask_thmin
	$G_EASE2 r.null null=0 map=${smap}_mask_thmax
	$G g.region vect=reg_$s1 res=0.009 -a
	$G r.proj location=EASE2 in=${smap}_mask_thmin output=${smap}_mask_thmin -n res=0.009 mem=4096 --o
	$G r.proj location=EASE2 in=${smap}_mask_thmax output=${smap}_mask_thmax -n res=0.009 mem=4096 --o
done

# Now computing temporal means of thresholded SMAP series

first=$(head -n +1 $OUTFILE)
first_s1=$(echo $first|cut -d\| -f1|sed -e 's/-/_/g')
first_smap=$(echo $first|cut -d\| -f3|sed -e 's/-/_/g')
$G_EASE2 g.region vect=reg_$first_s1

cat >$MAPCALC <<EOF
smap_mean_thmin=float(${first_smap}_thmin)
smap_mean_thmax=float(${first_smap}_thmax)
smap_contrib_thmin=float(${first_smap}_mask_thmin)
smap_contrib_thmax=float(${first_smap}_mask_thmax)
EOF
$G_EASE2 r.mapcalc file=$MAPCALC --o

for smapfile in $(tail -n +2 $OUTFILE|cut -d\| -f3|sed -e 's/-/_/g')
do
	cat >$MAPCALC <<EOF
new1=smap_contrib_thmin+${smapfile}_mask_thmin
smap_contrib_thmin=new1
new2=smap_contrib_thmax+${smapfile}_mask_thmax
smap_contrib_thmax=new2
new3=smap_mean_thmin+${smapfile}_thmin
smap_mean_thmin=new3
new4=smap_mean_thmax+${smapfile}_thmax
smap_mean_thmax=new4
EOF
	$G_EASE2 r.mapcalc file=$MAPCALC --o
done

	cat >$MAPCALC <<EOF
new1=smap_mean_thmin/smap_contrib_thmin
smap_mean_thmin=new1
new2=smap_mean_thmax/smap_contrib_thmax
smap_mean_thmax=new2
EOF
	$G_EASE2 r.mapcalc file=$MAPCALC --o

# Now projectiing resulting masks and thresholded means in WGS84

list=$($G g.list type=vect pat='reg_*' sep=,)
$G g.region vect=$list res=0.009 -a
$G r.proj location=EASE2 in=smap_mean_thmin out=smap_mean_thmin -n mem=4096 res=0.009 --o
$G r.proj location=EASE2 in=smap_mean_thmax out=smap_mean_thmax -n mem=4096 res=0.009 --o

# Now rescan S1 images and do almost the same, but mask by using SMAP thresholded masks

for line in $(cat $OUTFILE)
do
	s1file=$(echo $line|cut -d\| -f1)
	s1=$(echo $s1file|sed -e 's/-/_/g')
	smapfile=$(echo $line|cut -d\| -f3)
	smap=$(echo $smapfile|sed -e 's/-/_/g')
	$G g.region rast=${s1} res=0.009 -a
	$G r.resamp.stats in=${s1} out=r_${s1} method=average --o

	cat >$MAPCALC <<EOF
${s1}_thmin=if(${smap}_mask_thmin,r_$s1,0)
${s1}_mask_thmin=if(${smap}_mask_thmin,1,0)
EOF
	$G r.mapcalc file=$MAPCALC --o

	cat >$MAPCALC <<EOF
${s1}_thmax=if(${smap}_mask_thmax,r_$s1,0)
${s1}_mask_thmax=if(${smap}_mask_thmax,1,0)
EOF
	$G r.mapcalc file=$MAPCALC --o
done

first=$(head -n +1 $OUTFILE)
first_s1=$(echo $first|cut -d\| -f1|sed -e 's/-/_/g')
first_smap=$(echo $first|cut -d\| -f3|sed -e 's/-/_/g')
$G g.region rast=$first_s1
cat >$MAPCALC <<EOF
s1_mean_thmin=float(${first_s1}_thmin)
s1_mean_thmax=float(${first_s1}_thmax)
s1_contrib_thmin=float(${first_s1}_mask_thmin)
s1_contrib_thmax=float(${first_s1}_mask_thmax)
EOF
$G r.mapcalc file=$MAPCALC --o

for s1file in $(tail -n +2 $OUTFILE|cut -d\| -f1|sed -e 's/-/_/g')
do
	cat >$MAPCALC <<EOF
new1=s1_contrib_thmin+${s1file}_mask_thmin
s1_contrib_thmin=new1
new4=s1_mean_thmax+${s1file}_thmax
s1_mean_thmax=new4
EOF
	$G r.mapcalc file=$MAPCALC --o
done

	cat >$MAPCALC <<EOF
new1=s1_mean_thmin/s1_contrib_thmin
s1_mean_thmin=new1
new2=s1_mean_thmax/s1_contrib_thmax
s1_mean_thmax=new2
EOF
	$G r.mapcalc file=$MAPCALC --o

# Output final maps, finally

OUTDIR=$SM_TOOLS_OUTPUTDIR/$S1_TAG/$REFERENCE
mkdir -p $OUTDIR >/dev/null || true
$G r.out.gdal in=s1_mean_thmin out=$OUTDIR/s1_mean_thmin type=Float32 format=ENVI --o -f
$G r.out.gdal in=s1_contrib_thmin out=$OUTDIR/s1_contrib_thmin type=Float32 format=ENVI --o -f
$G r.out.gdal in=s1_mean_thmax out=$OUTDIR/s1_mean_thmax type=Float32 format=ENVI --o -f
$G r.out.gdal in=s1_contrib_thmax out=$OUTDIR/s1_contrib_thmax type=Float32 format=ENVI --o -f
$G r.out.gdal in=smap_mean_thmin out=$OUTDIR/smap_mean_thmin type=Float32 format=ENVI --o -f
$G_EASE2 r.out.gdal in=smap_contrib_thmin out=$OUTDIR/smap_contrib_thmin type=Float32 format=ENVI --o -f
$G r.out.gdal in=smap_mean_thmax out=$OUTDIR/smap_mean_thmax type=Float32 format=ENVI --o -f
$G_EASE2 r.out.gdal in=smap_contrib_thmax out=$OUTDIR/smap_contrib_thmax type=Float32 format=ENVI --o -f


#rm -f $OUTFILE $MAPCALC
#$G_CLEAN
