#
# database.t - test script for SSM_Land::Database
#
# Copyright (C) 2017, 2018 Francesco P. Lovergine <francesco.lovergine@cnr.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2, 
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

use strict;
use warnings;

use v5.18;

use Test::More;

use FindBin qw/$Bin/;
use lib ("/usr/local/sm-tools/lib/perl", "$Bin/../lib");
use Scalar::Util qw(looks_like_number);

BEGIN { 
	
use_ok('SSM_Tools::Database');

};

#########################

# Insert your test code below, the Test::More module is use()ed here so read
# its man page ( perldoc Test::More ) for help writing this test script.

$SSM_Tools::Database::Verbose = 1;

#
# initialize an empty test database
#
my $dbname = "test.splite";
eval {
	system("spatialite", "-init", "$Bin/../sql/build.sql", $dbname, ".quit");
};
die $@ if $@;

my $db = SSM_Tools::Database->new($dbname); 

ok(defined $db, 'new() returned something'); 
ok($db->isa("SSM_Tools::Database"), "and it is the right class");


# _timestamp test
my $t = $db->_timestamp();
ok($t =~ m/^(\d+)-(\d+)-(\d+) (\d+):(\d+):(\d+)$/, "Check _timestamp(now)");

# _depth tests
my ($str, $val, @d);
my %test = (
	" 2d5T5d6" => [2.5, 5.6, undef],
	"2 "       => [2, undef, undef],
	"T2" 	   => [undef, 2, undef],
	"T2d5"     => [undef, 2.5, undef],
	"2d50T"    => [2.5, undef, undef],
	"2:0"      => [2, undef, 0],
	"2d5T5d1:1 " => [2.5, 5.1, 1],
	":1"	   => [undef, undef, 1],
);
while (($str, $a) = each %test) {
	my @d = $db->_depth($str);
	ok($d[0] == $a->[0], "Check _depth $str 0") if defined $a->[0] && looks_like_number($a->[0]);
	ok($d[1] == $a->[1], "Check _depth $str 1") if defined $a->[1] && looks_like_number($a->[1]);
	ok($d[2] == $a->[2], "Check _depth $str 2") if defined $a->[2] && looks_like_number($a->[2]);

	ok($d[0] eq $a->[0], "Check _depth $str 3") if defined $a->[0] && !looks_like_number($a->[0]);
	ok($d[1] eq $a->[1], "Check _depth $str 4") if defined $a->[1] && !looks_like_number($a->[1]);
	ok($d[2] eq $a->[2], "Check _depth $str 5") if defined $a->[2] && !looks_like_number($a->[2]);
}

is($db->_trim("  string  "),"string", "Checking trim 1");
is($db->_trim("  string"),"string", "Checking trim 2");
is($db->_trim("string  \t\n"),"string", "Checking trim 3");

is($db->to_depth("temp",2.41,5.0), "temp_2d41T5d00", "Checking to_depth");

is($db->correct_temperature(432), "3.2", "Checking correct_temperature" );

# normalize_datetime tests
is($db->normalize_datetime(qq/2017-07-20 14:00/), qq/2017-07-20 14:00:00/,"Check normalize_datetime() 1");
is($db->normalize_datetime(qq/17-07-20 14:00:00/), qq/2017-07-20 14:00:00/,"Check normalize_datetime() 2");
is($db->normalize_datetime(qq/2017-07-20 14/), qq/2017-07-20 14:00:00/,"Check normalize_datetime() 3");
is($db->normalize_datetime(qq/2017-07-20/), qq/2017-07-20 00:00:00/,"Check normalize_datetime() 4");
is($db->normalize_datetime(qq/20170720/), qq/2017-07-20 00:00:00/,"Check normalize_datetime() 5");

# adjust_datetime_for_tz tests
is($db->adjust_datetime_for_tz(qq/2017-03-08 14:30:00/,'Europe/Rome'), qq/2017-03-08 13:30:00/, "Check adjust_datetime_for_tz() 1");
is($db->adjust_datetime_for_tz(qq/2017-03-08 14:30/,'Europe/Rome'), qq/2017-03-08 13:30:00/, "Check adjust_datetime_for_tz() 2");
is($db->adjust_datetime_for_tz(qq/2017-06-08 14:30/,'Europe/Rome'), qq/2017-06-08 12:30:00/, "Check adjust_datetime_for_tz() 3");
is($db->adjust_datetime_for_tz(qq/2017-03-08 14:30/,'UTC'), qq/2017-03-08 14:30:00/, "Check adjust_datetime_for_tz() 4");
is($db->adjust_datetime_for_tz(qq/2017-03-08 14:30:00/,'+01:00'), qq/2017-03-08 13:30:00/, "Check adjust_datetime_for_tz() 5");
is($db->adjust_datetime_for_tz(qq/2017-06-08 14:30:00/,'+01:00'), qq/2017-06-08 13:30:00/, "Check adjust_datetime_for_tz() 6");
is($db->adjust_datetime_for_tz(qq/2017-06-08 12:30:00/,'CST6CDT'), qq/2017-06-08 17:30:00/, "Check adjust_datetime_for_tz() 7");

# inject_ground_data tests
ok($db->inject_ground_data('segezia','segezia',{
	timestamp => '2017-03-08 14:30',
	ec => 0,
	a => 1.56,
	b => 0.4,
	long => 16.65734,
	lat => 41.56634,
	sensorid => 'Sg1:y12',
	rainfall_20 => 0,
	sm_0T5 => 134.0,
	temp_3 => 24.0,
}), 'Check inject_ground_data() 1');

ok($db->inject_ground_data('segezia','segezia',{
	timestamp => '2017-03-08 14:30',
	ec => 0,
	a => 1.56,
	b => 0.4,
	long => 16.65734,
	lat => 41.56634,
	sensorid => 'Sg1:y12',
	rainfall_2 => 0,
	sm_2T5 => 134.0,
	temp_4 => 24.0,
}), 'Check inject_ground_data() 2');

ok($db->inject_ground_data('segezia','segezia',{
	timestamp => '2017-03-08 14:35',
	ec => 14.2,
	a => 1.56,
	b => 0.4,
	long => 16.65734,
	lat => 41.56634,
	sensorid => 'Sg1:y12',
	rainfall_2 => 0,
	'sm_2T5:0' => 133.0,
	'sm_2T5:1' => 134.0,
	temp_4 => 24.0,
}), 'Check inject_ground_data() 3');

ok($db->inject_ground_data('segezia','segezia',{
	timestamp => '2017-03-08 14:35',
	ec => 14.2,
	a => 1.56,
	b => 0.4,
	long => 16.65734,
	lat => 41.56634,
	sensorid => 'Sg1:y12',
	rainfall_2 => 0,
	'sm_2T5:0' => 155.0,
	'sm_2T5:1' => 134.0,
	temp_4 => 24.0,
}), 'Check if inject_ground_data() ignores re-injection');

# get_sensors() test
my %sensors = $db->get_sensors();
cmp_ok($sensors{'sg1:y12'}->{a}, '==', 1.56, 'Check sensor (a)');
cmp_ok($sensors{'sg1:y12'}->{b}, '==', 0.4, 'Check sensor (b)');
cmp_ok($sensors{'sg1:y12'}->{longitude}, '==', 16.65734, 'Check sensor (longitude)');
cmp_ok($sensors{'sg1:y12'}->{latitude}, '==', 41.56634, 'Check sensor (latitude)');
cmp_ok($sensors{'sg1:y12'}->{network}, 'eq', 'segezia', 'Check sensor (network)');

cmp_ok($db->get_sites(),'eq','1','Check sites list');
cmp_ok($db->get_networks(),'eq','1','Check networks list');

# db method final tests
$db->commit;
$db->close;

unlink($dbname) if 0;

done_testing();

