#!/bin/sh
#
# Create a default mapfile for Mapserver starting from a group of rasters as
# produced by SMOSAR processing chain.
#
# Usage: create_mapserver_mapfile.sh  [-r|-m|-s] asc|desc
#
# The argument string is mandatory. Use -rms options to force computation of different 
# type of mosaics based on time ranges.
#

set -e

ROOT=/data/mapserver
ECHO="/bin/echo -e"

MAPFILE=default.map

REGEN=no
MONTHLY=no
SEASONAL=no

FULL=yes

USAGE="usage: $0: [-r|-m|-s] asc|desc\n\t-r (re)generate base maps\n\t-m (re)generate monthly mosaics\n\t-s (re)generate seasonal mosaics\n"
while getopts rms opt
do
    case $opt in
    r)   REGEN=yes;;
    m)   MONTHLY=yes;;
	s)	 SEASONAL=yes;;
    ?)   $ECHO $USAGE
         exit 2;;
    esac
done
shift $(($OPTIND - 1))

if [ $# -ne 1 ]
then
    $ECHO $USAGE
	exit 1
fi

case "$1" in
	desc|DESC)
		DIR=desc
		LABEL=DESC
		;;
	asc|ASC)
		DIR=asc
		LABEL=ASC
		;;
	*)
		$ECHO $USAGE
		exit 2
		;;
esac

cd $ROOT

SRCDIR=orig
OUTDIR=data

gdal_extent() {
 if [ -z "$1" ]; then 
  echo "Missing arguments. Syntax:"
  echo "  gdal_extent <input_raster>"
     return
 fi
 EXTENT=$(gdalinfo $1 |\
  grep "Upper Left\|Lower Right" |\
  sed "s/Upper Left  //g;s/Lower Right //g;s/).*//g" |\
  tr "\n" " " |\
  sed 's/ *$//g' |\
  tr -d "[(,]")
 EXTENT=`echo $EXTENT | awk '{print $1 " " $4 " " $3 " " $2}'`
 echo -n "$EXTENT"
}

ogr_extent() {
 if [ -z "$1" ]; then 
  echo "Missing arguments. Syntax:"
  echo "  ogr_extent <input_vector>"
     return
 fi
 EXTENT=$(ogrinfo -al -so $1 |\
  grep Extent |\
  sed 's/Extent: //g' |\
  sed 's/(//g' |\
  sed 's/)//g' |\
  sed 's/ - /, /g')
 EXTENT=`echo $EXTENT | awk -F ',' '{print $1 " " $2 " " $3 " " $4}'`
 echo -n "$EXTENT"
}







if [ "$REGEN" != "no" ]; then

# Initial reprojection in Webmercator and translation into GTiff files.
if [  $(ls $SRCDIR/$DIR/*_mean.tif 2>/dev/null|head -1|wc -l) -eq 0 ]
then
	for p in $SRCDIR/$DIR/*_mean $SRCDIR/$DIR/*_stddev
	do
		f=$(basename $p)
		gdalwarp -q -overwrite -of GTiff -t_srs EPSG:3857 -tr 500 500 $p $SRCDIR/$DIR/$f.tif
	done
fi

# Prepare RGBA TIF and pyramids for the previous files
for p in $SRCDIR/$DIR/*_mean.tif
do
	f=$(basename $p)
	gdaldem color-relief -alpha -q $p colors.txt $OUTDIR/$DIR/$f 2>/dev/null >/dev/null
	gdaladdo -r average $OUTDIR/$DIR/$f 2 4 8 16 32 64 >/dev/null
done
for p in $SRCDIR/$DIR/*_stddev.tif
do
	f=$(basename $p)
	gdaldem color-relief -alpha -q $p colors_stddev.txt $OUTDIR/$DIR/$f 2>/dev/null >/dev/null
	gdaladdo -r average $OUTDIR/$DIR/$f 2 4 8 16 32 64 >/dev/null
done

# Find dates for mosaics
start_date=$(for i in $SRCDIR/$DIR/*_mean.tif; do echo $i|cut -d_ -f4; done|cut -dT -f1|sort|head -1)
end_date=$(for i in $SRCDIR/$DIR/*_mean.tif; do echo $i|cut -d_ -f4; done|cut -dT -f1|sort|tail -1)

# Mosaics of tracks on a 6 days basis
mkdir -p $OUTDIR/$DIR/mosaics 2>/dev/null >/dev/null || true
while [ $start_date -lt $end_date ]
do
	list=$(for i in $(seq 0 5); do date --date="$start_date + $i day" +'%Y%m%d'; done|sed -e 's/$/,/'|perl -00pe 's/,(?!.*,)//s')
	regexp=$(echo $list|sed -e 's/, /|/g')
	images=$(ls $OUTDIR/$DIR/*_mean.tif|egrep "$regexp")
	gdalwarp -q $images $OUTDIR/$DIR/mosaics/mosaic_mean_$start_date.tif 2>/dev/null >/dev/null
	[ -f $OUTDIR/$DIR/mosaics/mosaic_mean_$start_date.tif ] && gdaladdo -r average $OUTDIR/$DIR/mosaics/mosaic_mean_$start_date.tif 2 4 8 16 32 64 128 >/dev/null
	images=$(ls $OUTDIR/$DIR/*_stddev.tif|egrep "$regexp")
	gdalwarp -q $images $OUTDIR/$DIR/mosaics/mosaic_stddev_$start_date.tif 2>/dev/null >/dev/null
	[ -f $OUTDIR/$DIR/mosaics/mosaic_stddev_$start_date.tif ] && gdaladdo -r average $OUTDIR/$DIR/mosaics/mosaic_stddev_$start_date.tif 2 4 8 16 32 64 128 >/dev/null
	start_date=$(date --date="$start_date + 6 day" +"%Y%m%d")
done


fi # $REGEN?






# Create tindex for tracks

for t in $(ls $OUTDIR/$DIR/*_mean.tif|cut -d_ -f10,11|cut -d. -f 1|sort|uniq)
do
	rm -f ${DIR}_doq_${t}_mean_index.* ${DIR}_doq_${t}_stddev_index.*
	gdaltindex ${DIR}_doq_${t}_mean_index.shp $OUTDIR/$DIR/*${t}*_mean.tif 2>/dev/null >/dev/null
	gdaltindex ${DIR}_doq_${t}_stddev_index.shp $OUTDIR/$DIR/*${t}*_stddev.tif 2>/dev/null >/dev/null

	ogrinfo ${DIR}_doq_${t}_mean_index.shp -sql "alter table ${DIR}_doq_${t}_mean_index add column time text" 2>/dev/null >/dev/null
	ogrinfo ${DIR}_doq_${t}_stddev_index.shp -sql "alter table ${DIR}_doq_${t}_stddev_index add column time text" 2>/dev/null >/dev/null

	# Populate indexes with dates for every track for WMS-T services

	for f in $OUTDIR/$DIR/*${t}*_mean.tif
	do
		d=$(echo $f|cut -d\/ -f3|sed -e 's/.tif//'|cut -d_ -f9|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
		ogrinfo ${DIR}_doq_${t}_mean_index.shp -dialect sqlite -sql "update ${DIR}_doq_${t}_mean_index set time='$d' where location='$f'" 2>/dev/null >/dev/null
	done
	for f in $OUTDIR/$DIR/*${t}*_stddev.tif
	do
		d=$(echo $f|cut -d\/ -f3|sed -e 's/.tif//'|cut -d_ -f9|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
		ogrinfo ${DIR}_doq_${t}_stddev_index.shp -dialect sqlite -sql "update ${DIR}_doq_${t}_stddev_index set time='$d' where location='$f'" 2>/dev/null >/dev/null
	done
done







# Create tindex for mosaics

rm -f ${DIR}_doq_mean_index.* ${DIR}_doq_stddev_index.*
gdaltindex ${DIR}_doq_mean_index.shp $OUTDIR/$DIR/mosaics/mosaic_mean_*.tif 2>/dev/null >/dev/null
gdaltindex ${DIR}_doq_stddev_index.shp $OUTDIR/$DIR/mosaics/mosaic_stddev_*.tif 2>/dev/null >/dev/null

ogrinfo ${DIR}_doq_mean_index.shp -sql "alter table ${DIR}_doq_mean_index add column time text" 2>/dev/null >/dev/null
ogrinfo ${DIR}_doq_stddev_index.shp -sql "alter table ${DIR}_doq_stddev_index add column time text" 2>/dev/null >/dev/null

# Populate index with dates of every mosaic, for WMS-T service

for f in $OUTDIR/$DIR/mosaics/mosaic_mean_*.tif
do
	d=$(echo $f|cut -d\/ -f4|sed -e 's/.tif//'|cut -d_ -f3|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
	ogrinfo ${DIR}_doq_mean_index.shp -dialect sqlite -sql "update ${DIR}_doq_mean_index set time='$d' where location='$f'" 2>/dev/null >/dev/null
done
for f in $OUTDIR/$DIR/mosaics/mosaic_stddev_*.tif
do
	d=$(echo $f|cut -d\/ -f4|sed -e 's/.tif//'|cut -d_ -f3|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
	ogrinfo ${DIR}_doq_stddev_index.shp -dialect sqlite -sql "update ${DIR}_doq_stddev_index set time='$d' where location='$f'" 2>/dev/null >/dev/null
done







#
# This is used to generate maps and mosaics for monthly and seasonal maps
#


if [ "$MONTHLY" != "no" ]; then

# Mosaics of tracks month date
mkdir -p $OUTDIR/$DIR/monthly 2>/dev/null >/dev/null || true

# Initial reprojection in Webmercator and translation into GTiff files.
if [  $(ls $SRCDIR/$DIR/monthly/*_mean.img.tif 2>/dev/null|head -1|wc -l) -eq 0 ]
then
	for p in $SRCDIR/$DIR/monthly/*_mean.img $SRCDIR/$DIR/monthly/*_std.img
	do
		f=$(basename $p)
		gdalwarp -q -overwrite -of GTiff -t_srs EPSG:3857 -tr 500 500 $p $SRCDIR/$DIR/monthly/$f.tif
	done
fi

# Prepare RGBA TIF and pyramids for the previous files
for p in $SRCDIR/$DIR/monthly/*_mean.img.tif
do
	f=$(basename $p)
	gdaldem color-relief -alpha -q $p colors.txt $OUTDIR/$DIR/monthly/$f 2>/dev/null >/dev/null
	gdaladdo -r average $OUTDIR/$DIR/monthly/$f 2 4 8 16 32 64 >/dev/null
done
for p in $SRCDIR/$DIR/monthly/*_std.img.tif
do
	f=$(basename $p)
	gdaldem color-relief -alpha -q $p colors_stddev.txt $OUTDIR/$DIR/monthly/$f 2>/dev/null >/dev/null
	gdaladdo -r average $OUTDIR/$DIR/monthly/$f 2 4 8 16 32 64 >/dev/null
done

# Find dates for mosaics
dates=$(for i in $(ls $SRCDIR/$DIR/monthly/*_mean.img.tif); do echo $(basename $i); done|cut -d_ -f11|sort|uniq)

for d in $(echo $dates)
do
	images=$(ls $OUTDIR/$DIR/monthly/*_mean.img.tif|grep $d)
	gdalwarp -q $images $OUTDIR/$DIR/monthly/mosaic_mean_$d.tif 2>/dev/null >/dev/null
	[ -f $OUTDIR/$DIR/monthly/mosaic_mean_$d.tif ] && gdaladdo -r average $OUTDIR/$DIR/monthly/mosaic_mean_$d.tif 2 4 8 16 32 64 128 >/dev/null
	images=$(ls $OUTDIR/$DIR/monthly/*_std.img.tif|grep $d)
	gdalwarp -q $images $OUTDIR/$DIR/monthly/mosaic_stddev_$d.tif 2>/dev/null >/dev/null
	[ -f $OUTDIR/$DIR/monthly/mosaic_stddev_$d.tif ] && gdaladdo -r average $OUTDIR/$DIR/monthly/mosaic_stddev_$d.tif 2 4 8 16 32 64 128 >/dev/null
done


fi # $MONTHLY

# Create tindex
rm -f ${DIR}_doq_monthly_mean_index.* ${DIR}_doq_monthly_stddev_index.*
gdaltindex ${DIR}_doq_monthly_mean_index.shp $OUTDIR/$DIR/monthly/mosaic_mean_*.tif 2>/dev/null >/dev/null
gdaltindex ${DIR}_doq_monthly_stddev_index.shp $OUTDIR/$DIR/monthly/mosaic_stddev_*.tif 2>/dev/null >/dev/null

ogrinfo ${DIR}_doq_monthly_mean_index.shp -sql "alter table ${DIR}_doq_monthly_mean_index add column time text" 2>/dev/null >/dev/null
ogrinfo ${DIR}_doq_monthly_stddev_index.shp -sql "alter table ${DIR}_doq_monthly_stddev_index add column time text" 2>/dev/null >/dev/null

# Populate index with dates of every mosaic, for WMS-T service

for f in $OUTDIR/$DIR/monthly/mosaic_mean_*.tif
do
	d=$(echo $f|cut -d\/ -f4|sed -e 's/.tif//'|cut -d_ -f3|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
	ogrinfo ${DIR}_doq_monthly_mean_index.shp -dialect sqlite -sql "update ${DIR}_doq_monthly_mean_index set time='$d' where location='$f'" 2>/dev/null >/dev/null
done
for f in $OUTDIR/$DIR/monthly/mosaic_stddev_*.tif
do
	d=$(echo $f|cut -d\/ -f4|sed -e 's/.tif//'|cut -d_ -f3|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
	ogrinfo ${DIR}_doq_monthly_stddev_index.shp -dialect sqlite -sql "update ${DIR}_doq_monthly_stddev_index set time='$d' where location='$f'" 2>/dev/null >/dev/null
done







if [ "$SEASONAL" != "no" ]; then

# Mosaics of tracks month date
mkdir -p $OUTDIR/$DIR/seasonal 2>/dev/null >/dev/null || true

# Initial reprojection in Webmercator and translation into GTiff files.
if [  $(ls $SRCDIR/$DIR/seasonal/*_mean.img.tif 2>/dev/null|head -1|wc -l) -eq 0 ]
then
	for p in $SRCDIR/$DIR/seasonal/*_mean.img $SRCDIR/$DIR/seasonal/*_std.img
	do
		f=$(basename $p)
		gdalwarp -q -overwrite -of GTiff -t_srs EPSG:3857 -tr 500 500 $p $SRCDIR/$DIR/seasonal/$f.tif
	done
fi

# Prepare RGBA TIF and pyramids for the previous files
for p in $SRCDIR/$DIR/seasonal/*_mean.img.tif
do
	f=$(basename $p)
	gdaldem color-relief -alpha -q $p colors.txt $OUTDIR/$DIR/seasonal/$f 2>/dev/null >/dev/null
	gdaladdo -r average $OUTDIR/$DIR/seasonal/$f 2 4 8 16 32 64 >/dev/null
done
for p in $SRCDIR/$DIR/seasonal/*_std.img.tif
do
	f=$(basename $p)
	gdaldem color-relief -alpha -q $p colors_stddev.txt $OUTDIR/$DIR/seasonal/$f 2>/dev/null >/dev/null
	gdaladdo -r average $OUTDIR/$DIR/seasonal/$f 2 4 8 16 32 64 >/dev/null
done

for s in winter spring summer autumn
do
	# Find dates for mosaics
	years=$(for i in $(ls $SRCDIR/$DIR/seasonal/*$s*_mean.img.tif); do echo $(basename $i); done|cut -d_ -f11|sort|uniq)
	for d in $(echo $years)
	do
		images=$(ls $OUTDIR/$DIR/seasonal/*$s*_mean.img.tif|grep $d)
		gdalwarp -q $images $OUTDIR/$DIR/seasonal/mosaic_mean_${s}_$d.tif 2>/dev/null >/dev/null
		[ -f $OUTDIR/$DIR/seasonal/mosaic_mean_${s}_$d.tif ] && gdaladdo -r average $OUTDIR/$DIR/seasonal/mosaic_mean_${s}_$d.tif 2 4 8 16 32 64 128 >/dev/null
		images=$(ls $OUTDIR/$DIR/seasonal/*$s*_std.img.tif|grep $d)
		gdalwarp -q $images $OUTDIR/$DIR/seasonal/mosaic_stddev_${s}_$d.tif 2>/dev/null >/dev/null
		[ -f $OUTDIR/$DIR/seasonal/mosaic_stddev_${s}_$d.tif ] && gdaladdo -r average $OUTDIR/$DIR/seasonal/mosaic_stddev_${s}_$d.tif 2 4 8 16 32 64 128 >/dev/null
	done
done

fi # SEASONAL

# Create tindex
rm -f ${DIR}_doq_seasonal_mean_index.* ${DIR}_doq_seasonal_stddev_index.*
gdaltindex ${DIR}_doq_seasonal_mean_index.shp $OUTDIR/$DIR/seasonal/mosaic_mean_*.tif 2>/dev/null >/dev/null
gdaltindex ${DIR}_doq_seasonal_stddev_index.shp $OUTDIR/$DIR/seasonal/mosaic_stddev_*.tif 2>/dev/null >/dev/null






# Find start/end dates for the WMS-T layers

MOSAICS_STARTTIME=$(ls $OUTDIR/$DIR/mosaics/*mean*tif|cut -d\/ -f4|sed -e 's/.tif//'|cut -d_ -f3|head -n +1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
MOSAICS_ENDTIME=$(ls $OUTDIR/$DIR/mosaics/*mean*tif|cut -d\/ -f4|sed -e 's/.tif//'|cut -d_ -f3|tail -n -1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')

MONTHLY_MOSAICS_STARTTIME=$(ls $OUTDIR/$DIR/monthly/mosaic_mean*tif|cut -d\/ -f4|sed -e 's/.tif//'|cut -d_ -f3|head -n +1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
MONTHLY_MOSAICS_ENDTIME=$(ls $OUTDIR/$DIR/monthly/mosaic_mean*tif|cut -d\/ -f4|sed -e 's/.tif//'|cut -d_ -f3|tail -n -1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')

# !!!!! Note that all paths are always relative to current ROOT, even for tindex 

E=$(ogr_extent desc_doq_mean_index.shp)
EU_E=$(ogr_extent data/eurasian.shp)

if [ ! -e $MAPFILE ] 
then

# Mapfile global prefix
cat >$MAPFILE <<EOF
MAP
NAME "SMOSAR_MAPS"
EXTENT $E
SIZE 1920 1080
UNITS meters

OUTPUTFORMAT
  NAME "png"
  DRIVER AGG/PNG
  MIMETYPE "image/png"
  IMAGEMODE RGB
  EXTENSION "png"
  FORMATOPTION "GAMMA=0.75"
  TRANSPARENT on
END

PROJECTION
	"init=epsg:3857"
END 

WEB
  IMAGEPATH "/var/www/html/mapserver/tmp/"
  IMAGEURL "/mapserver/tmp/"
  METADATA
    "wms_title"           "CNR WMS Server"
    "wms_onlineresource"  "http://150.145.93.9/wmsmap"
    "wms_srs"             "EPSG:3857"
	"wms_enable_request"  "*"
	"wms_abstract"		  "Multiple maps over Mediterranean area, created with SMOSAR algorithm for soil moisture retrieval, using Sentinel-1"
	"wms_contactelectronicmailaddress" "francesco.lovergine@cnr.it"
	"wms_contactfacsimiletelephone" "None"
	"wms_contactperson" "Francesco P Lovergine"
	"wms_contactorganization" "CNR-IREA"
	"wms_contactposition" "Researcher"
	"wms_contactvoicetelephone" "None"
  END
END

    LAYER 
        NAME "EU_BASINS"
        DATA "data/eurasian.shp"
        EXTENT $EU_E
        TYPE POLYGON
        STATUS on
        PROJECTION
            "init=epsg:3857"
        END 
        METADATA
            "wms_title" "EU major basins"
        END
        CLASS
            STYLE
                OUTLINECOLOR 0 0 0
            END
        END
    END

    INCLUDE 'asc_$MAPFILE'
    INCLUDE 'desc_$MAPFILE'

END
EOF

fi





cat >${DIR}_$MAPFILE <<EOF

	# WMS-T supported layer 

	LAYER
	  NAME "Six-days $LABEL Mean Mosaics"
	  METADATA
			"wms_title"    "Six-days $LABEL Mean Mosaics"
			"wms_timeextent" "$MOSAICS_STARTTIME/$MOSAICS_ENDTIME"
			"wms_timeitem" "time"
			"wms_timedefault" "$MOSAICS_STARTTIME"
			"wms_enable_request" "*"
			"wms_timeformat" "YYYY-MM-DD"
	  END
	  TYPE RASTER
	  TILEINDEX "${DIR}_doq_mean_index.shp"
      TILEITEM "location"
      PROJECTION
            "init=epsg:3857"
      END
	  STATUS on
	  FILTER ('[time]'='$MOSAICS_STARTTIME')
	END

	LAYER
	  NAME "Six-days $LABEL Stddev Mosaics"
	  METADATA
			"wms_title"    "Six-days $LABEL Stddev Mosaics"
			"wms_timeextent" "$MOSAICS_STARTTIME/$MOSAICS_ENDTIME"
			"wms_timeitem" "time"
			"wms_timedefault" "$MOSAICS_STARTTIME"
			"wms_enable_request" "*"
			"wms_timeformat" "YYYY-MM-DD"
	  END
	  TYPE RASTER
	  TILEINDEX "${DIR}_doq_stddev_index.shp"
      TILEITEM "location"
      PROJECTION
            "init=epsg:3857"
      END
	  STATUS on
	  FILTER ('[time]'='$MOSAICS_STARTTIME')
	END

	LAYER
	  NAME "Monthly $LABEL Mean Mosaics"
	  METADATA
		"wms_title"    "Monthly $LABEL Mean Mosaics"
		"wms_timeextent" "$MONTHLY_MOSAICS_STARTTIME/$MONTHLY_MOSAICS_ENDTIME"
		"wms_timeitem" "time"
		"wms_timedefault" "$MONTHLY_MOSAICS_STARTTIME"
		"wms_enable_request" "*"
		"wms_timeformat" "YYYY-MM-DD"
	  END
	  TYPE RASTER
	  TILEINDEX "${DIR}_doq_monthly_mean_index.shp"
      TILEITEM "location"
      PROJECTION
            "init=epsg:3857"
      END
	  STATUS on
	  FILTER ('[time]'='$MONTHLY_MOSAICS_STARTTIME')
	END

	LAYER
	  NAME "Monthly $LABEL Stddev Mosaics"
	  METADATA
		"wms_title"    "Monthly $LABEL Stddev Mosaics"
		"wms_timeextent" "$MONTHLY_MOSAICS_STARTTIME/$MONTHLY_MOSAICS_ENDTIME"
		"wms_timeitem" "time"
		"wms_timedefault" "$MONTHLY_MOSAICS_STARTTIME"
		"wms_enable_request" "*"
		"wms_timeformat" "YYYY-MM-DD"
	  END
	  TYPE RASTER
	  TILEINDEX "${DIR}_doq_monthly_stddev_index.shp"
      TILEITEM "location"
      PROJECTION
            "init=epsg:3857"
      END
	  STATUS on
	  FILTER ('[time]'='$MONTHLY_MOSAICS_STARTTIME')
	END

EOF

echo "START TRASACTION;"  >${DIR}.sql



# Here generate WMS-T service for every track


for t in $(ls $OUTDIR/$DIR/*_mean.tif|cut -d_ -f10,11|cut -d. -f 1|sort|uniq)
do
	START_TIME=$(ls $OUTDIR/$DIR/*$t*_mean.tif|cut -d_ -f9|cut -dT -f1|sort|uniq|head -n +1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
	END_TIME=$(ls $OUTDIR/$DIR/*$t*_mean.tif|cut -d_ -f9|cut -dT -f1|sort|uniq|tail -n -1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
	# Single date layers ssm
	if [ "$FULL" != "no" ]; then
		cat >>${DIR}_$MAPFILE <<EOF
	LAYER 
		NAME "${LABEL}_SSM_${t}"
	    METADATA
			"wms_title" "$LABEL SSM $t"
			"wms_timeextent" "$START_TIME/$END_TIME"
			"wms_timeitem" "time"
			"wms_timedefault" "$START_TIME"
			"wms_enable_request" "*"
			"wms_timeformat" "YYYY-MM-DD"
  	    END
		TYPE RASTER
	    TILEINDEX "${DIR}_doq_${t}_mean_index.shp"
        TILEITEM "location"
		STATUS on
		PROJECTION
			"init=epsg:3857"
		END 
		METADATA
		END
	  	FILTER ('[time]'='$START_TIME')
	END

EOF
	fi
	for p in $OUTDIR/$DIR/*$t*_mean.tif
	do
		f=$(basename $p .tif)
		d=$(echo $f|cut -d_ -f4|cut -dT -f1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
		echo "REPLACE INTO webgis VALUES('${LABEL}_SINGLE_MEAN','$d','$p','${LABEL}_SSM_${t}_$d');"  >>${DIR}.sql
	done

done

for t in $(ls $OUTDIR/$DIR/*_stddev.tif|cut -d_ -f10,11|cut -d. -f 1|sort|uniq)
do
	START_TIME=$(ls $OUTDIR/$DIR/*$t*_stddev.tif|cut -d_ -f9|cut -dT -f1|sort|uniq|head -n +1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
	END_TIME=$(ls $OUTDIR/$DIR/*$t*_stddev.tif|cut -d_ -f9|cut -dT -f1|sort|uniq|tail -n -1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
	# Single track layers stddev
	if [ "$FULL" != "no" ]; then
		cat >>${DIR}_$MAPFILE <<EOF
	LAYER 
		NAME "${LABEL}_SSM_SD_${t}"
	    METADATA
			"wms_title" "$LABEL SSM $t"
			"wms_timeextent" "$START_TIME/$END_TIME"
			"wms_timeitem" "time"
			"wms_timedefault" "$START_TIME"
			"wms_enable_request" "*"
			"wms_timeformat" "YYYY-MM-DD"
  	    END
		TYPE RASTER
	    TILEINDEX "${DIR}_doq_${t}_stddev_index.shp"
        TILEITEM "location"
		STATUS on
		PROJECTION
			"init=epsg:3857"
		END 
		METADATA
		END
	  	FILTER ('[time]'='$START_TIME')
	END

EOF
	fi
	for p in $OUTDIR/$DIR/*$t*_stddev.tif
	do
		f=$(basename $p .tif)
		d=$(echo $f|cut -d_ -f4|cut -dT -f1|perl -ne 'chomp; ($y,$m,$d) = unpack "A4A2A2", $_; print sprintf "%04d-%02d-%02d", $y, $m, $d;')
		echo "REPLACE INTO webgis VALUES('${LABEL}_SINGLE_SD','$d','$p','${LABEL}_SSM_SD_${t}_$d');"  >>${DIR}.sql
	done

done







# seasonal mosaic layers
for p in $OUTDIR/$DIR/seasonal/mosaic_mean_*.tif
do
	f=$(basename $p .tif)
	d=$(echo $f|cut -d_ -f4)
	s=$(echo $f|cut -d_ -f3|perl -ne 'chomp; print "\U$_\E\n";')
	e=$(gdal_extent $p)
	cat >>${DIR}_$MAPFILE <<EOF
	LAYER 
		NAME "MOSAIC_${LABEL}_SSM_${s}_$d"
		DATA "$p"
		EXTENT $e
		TYPE raster
		STATUS on
		PROJECTION
			"init=epsg:3857"
		END 
		METADATA
			"wms_title" "Seasonal mosaic $LABEL SSM $s $d"
		END
	END

EOF
	echo "REPLACE INTO webgis VALUES('${LABEL}_MOSAICSEASONAL_MEAN','$d-01-01','$p','MOSAIC_${LABEL}_SSM_${s}_$d');"  >>${DIR}.sql
done

# seasonal mosaic layers stddev
for p in $OUTDIR/$DIR/seasonal/mosaic_stddev_*.tif
do
	f=$(basename $p .tif)
	d=$(echo $f|cut -d_ -f4)
	s=$(echo $f|cut -d_ -f3|perl -ne 'chomp; print "\U$_\E\n";')
	e=$(gdal_extent $p)
	cat >>${DIR}_$MAPFILE <<EOF
	LAYER 
		NAME "MOSAIC_${LABEL}_SSM_SD_${s}_$d"
		DATA "$p"
		EXTENT $e
		TYPE raster
		STATUS on
		PROJECTION
			"init=epsg:3857"
		END 
		METADATA
			"wms_title" "Seasonal mosaic $LABEL SSM STDDEV $s $d"
		END
	END

EOF
	echo "REPLACE INTO webgis VALUES('${LABEL}_MOSAICSEASONAL_SD','$d-01-01','$p','MOSAIC_${LABEL}_SSM_SD_${s}_$d');"  >>${DIR}.sql
done

echo "COMMIT;"  >>${DIR}.sql

exit 0
